/*	
 *	laza.modal() :: adding modal window to any layer (typically 'body')
 *
 *	Copyright by Lazaworx
 *	http://www.lazaworx.com
 *	Author: Laszlo Molnar
 *
 *	Dual licensed under the MIT and GPL licenses.
 *	- http://www.opensource.org/licenses/mit-license.php
 *	- http://www.gnu.org/copyleft/gpl.html
 *
 *	Usage: $(element).modal( content, buttons, options );
 */
 
;(function($, $window) {
	'use strict';
		
	$.fn.modal = function(content, buttons, settings) {
		
		var self = $(this);
		
		if (typeof content === 'string') {
			content = $(document.createTextNode(content));
		}
		
		if (!(content instanceof $ && content.length)) {
			return;
		}
				
		if (!$.isArray(buttons)) { 
			settings = buttons; 
			buttons = null;
		}
		
		settings = $.extend( {}, $.fn.modal.defaults, settings );

		var ns = 'lmo_' + Math.floor(Math.random() * 10000),
		
			text = getTranslations($.fn.modal.text),
		
			modal = $('<div>', {
					'class': 	'modal'
				}).appendTo(self),
				
			popup = $('<div>', {
					'class': 	'window ' + settings['class'] || '',
					role: 		'dialog'
				}).appendTo(modal),
				
			closeBtn = $('<a>', {
					'class': 	'btn close',
					'title': 	text.closeWindow
				}).appendTo(popup),
				
			cont = $('<div>', {
					'class': 	'content'
				}).appendTo(popup),
				
			to = null,
			rto = null,
						
			setMaxHeight = function() {
					cont.css({
						overflow:	'auto',
						maxHeight:	$window.height() * 0.8 - (settings.title? (closeBtn.height() + 40) : 0)
					});
				},
				
			close = function() {
					to = clearTimeout(to);
					modal.destroyAllTooltips().animate({
						opacity: 0
					}, settings.speed, function() {
						modal.remove();
						$window.off('.' + ns);
					});
					return false;
				};
		
		modal.on('destroy', close);
				
		// Prepare
		modal.hide().fadeIn(settings.speed);
		
		closeBtn.add(modal).on('click', function(e) {
			if (e.target === this) {
				close();
			}
		});
		
		if (settings.title) {
			popup.addClass('has-header').prepend($('<header>', {
				text: settings.title
			}));
		}
		
		setMaxHeight();
		
		cont.append(content.show());
		
		if (buttons) {
			
			var i, 
				a, 
				
				btnCont = $('<div>', { 
					'class': 'buttons' 
				}).appendTo(cont),
				
				clickhandler = function(e) {
					e.preventDefault();
					var a = e.target;
					if ( $.isFunction(a.handler) ) {
						if (a.handler.call(this) !== false) {
							close();
						}
					}
					return false;
				};
			
			for (i = 0; i < buttons.length; i++) {
				a = $('<button>', {
						'class': 'button ' + buttons[i].c,
					html: buttons[i].t
				}).on('click', clickhandler).appendTo(btnCont);
				
				if ($.isFunction(buttons[i].h)) {
					a[0].handler = buttons[i].h;
				}
			}
		}
		
		if (settings.autoFade) {
			to = setTimeout(close, settings.autoFade);
		}
		
		$window.on('resize.' + ns, function() {
				clearTimeout(rto);
				rto = setTimeout(setMaxHeight, 20);
			});
		
		closeBtn.addTooltip();
		
		return this;
	};
	
	$.fn.modal.defaults = {
		speed: 		300,
		autoFade: 	0,
		width: 		400,
		type: 		'normal'
	};
	
	$.fn.modal.text = {
		closeWindow: 'Close window'
	};

})(jQuery, jQuery(window));
