/*
 *	init.js - initializing the skin, skin-specific utilities
 *	Author: Laza
 */

	/* 
	 *	Java imports
	 */

var JAFilter = Java.type("se.datadosen.jalbum.JAFilter"),
	Category = Java.type("se.datadosen.jalbum.Category"),
	Widgets = Java.type("se.datadosen.jalbum.Widgets"),
	JAlbumUtilities = Java.type("se.datadosen.jalbum.JAlbumUtilities"),
	DateRange = Java.type("se.datadosen.jalbum.DateRange"),
	FixedShapeFilter = Java.type("FixedShapeFilter"),
	File = Java.type("java.io.File"),
	SkinProperties = Java.type("se.datadosen.jalbum.SkinProperties"),
	MyJSONObject = Java.type("se.datadosen.util.MyJSONObject"),
	System = Java.type("java.lang.System"),
	Zip = Java.type("tiger.Zip"),
	AtomicInteger = Java.type("java.util.concurrent.atomic.AtomicInteger");

	// No multiple index pages :: setting up before loading Util
	engine.setRows(0);
	
	// URL encoding, Write UTF-8 are madatory because the JSON database is always written in this format
	engine.setUrlEncode(true);
	engine.setWriteUTF8(true);
	
	// Zip library
	var zip = new Zip(engine);
	engine.setUsing(zip);
	
	/* 
	 *	Global variables
	 */

	time = (new Date()).getTime();					// time
	today = (time / 86400000) | 0;					// today: number of days since 1970-01-01

	// Fixing wrong Facebook App ID
	if (facebookAppId.indexOf('E') !== -1) {
		print('Facebook App ID is in wrong format, please check it in the Settings!');
		facebookAppId = facebookAppId.split('E')[0].replace(/\./g, '');
	}
	
	// HTML attribute
	htmlAttr = (shareFacebook? 'prefix="og: http://ogp.me/ns# fb: http://ogp.me/ns/fb#" ' : '') + 'lang="' + lang + '"';
	
	// Page protocol
	pageProtocol = basePath.startsWith('https://')? 'https:' : 'http:';
	
	// Debug mode? Don't use the minified javascript libraries.
	min = (typeof debugMode !== UNDEF && !!debugMode)? '' : '.min';
	
	// Undefined
	UNDEF = 'undefined';
	
	// Grid classes
	folderGridClass = 'caption-' + folderCaptionPlacement + getGridLadder(folderCols);
	pageGridClass = getGridLadder(pageCols);
	thumbGridClass = 'caption-' + captionPlacement + getGridLadder(thumbCols);
	
	// Folder image :: this image is copied to the index file's folder with the given name, cropped to the given size
	// Can be used as splash image for example
	// GUI should contain a JTextField folderImageSize, otherwise uncomment this:
	// folderImageSize = "800x400";
	folderImageFileName = 'folderimage.jpg';
	folderImageDims = (heroFullWidth? 1600 : ((parseFloat(maxPageWidth) || 67.5) * 16)) + 'x' + folderImageHeight;
	
	// Folder thumbnail :: placed along the folder image
	// Can be used for sharing over social sites as page thumbnail
	boxPad = (function() {
			switch (boxPadding) {
				case 'none': 	return 0;
				case 'small':	return 4;
				case 'medium':	return 8;
				case 'large':	return 12;
				case 'x-large':	return 20;
			}
			return 0;
		}());
	folderThumbFileName = 'folderthumb.jpg';
	folderThumbDims = (function() {
			var w,
				h,
				mpw = 980 * zoom;
				
			if (folderCols < 2) {
				w = mpw - 2 * boxPad;
			} else {
				var fc2 = Math.max(Math.floor(folderCols * 0.75), 1);
				w = Math.round((mpw - (fc2 - 1) * 4) / fc2) - 2 * boxPad;
			}
			
			if (fixedShapeFolderThumbs) {
				if (folderThumbAspectRatio) {
					var ar = folderThumbAspectRatio.split(':');
					h = Math.round(w * ar[1] / ar[0]);
				} else {
					h = Math.round(w * maxThumbHeight / maxThumbWidth);
				}
			} else {
				h = w;
			}
			
			return w + 'x' + h;
		}());
		
	// album description processed
	albumCaption = getProcessed(albumDescription);		
	
	// html formatted album info block 
	albumInfo = (function() {
			var s = '';
			
			if (typeof writer !== UNDEF) {
				s += '<div class="author"><span>' + getText('author') + '</span> '+ writer + '</div>';
			}
			
			if (typeof copyright !== UNDEF) {
				s += '<div class="copyright"><span>' + getText('copyright') + '</span> ' + copyright + '</div>';
			}
			
			if (typeof currentDate !== UNDEF && showModifiedDate) {
				s += '<div class="modifieddate"><span>' + getText('modified') + '</span> ' + currentDate + '</div>';
			}
			
			if (showImageCount) {
				s += getCounts(rootFolder, true, 'div', 'counts');
			}
			
			return s;
		}());
	
	albumThumbUrl = urlEncodeFix(basePath + folderThumbFileName);
		
	credits = '';
	
	// Feature detection
	_useFacebookBox = !!useFacebookBox && facebookAppId && facebookPageId;
	_useFacebookCommenting = !!facebookCommenting && facebookAppId;
	_useFacebook = _useFacebookBox || shareFacebook || _useFacebookCommenting;
	_useGapi = true;
	_useGoogleBox = useGoogleBox && googleBadgeId;
	_usePinterest = pinItButton;
	_useTopNavigation = topNavigationIncludeFolders || topNavigationIncludePages || topNavigationIncludeWebLocations;
	_useDisqusCommenting = !!disqusCommenting && disqusAppId;
	_useShop = !!showShop && shopId;
	_useFeedback = !!showFeedback && feedbackEmail && feedbackTemplate;
	_useMap = (showMapSection || showMap) && googleApiKey;
	_useTagCloud = tagCloudSource !== 'none';
	_useSearchNew = searchNewSource !== 'none';
	_usePhotodata = !!showPhotoData && photoDataTemplate;
	_useRegions = !!showRegions;
	_useMostPhotos = !!useMostphotos;
	_useTagCloudBox = tagCloudSource !== 'none' && tagCloudFields;
	_useSearch = !!useSearch && searchFields;
	_useSlider = heroType === 'randomImage' || heroType === 'allImages';
	_anyVr = false;
	_anyShares =
			facebookLike ||
			twitterTweet ||
			googlePlus ||
			tumblrButton ||
			pinItButton ||
			shareFacebook || 
			shareTwitter || 
			shareGplus ||
			sharePinterest ||
			shareLinkedIn ||
			shareDigg || 
			shareStumbleupon || 
			shareReddit ||
			shareTumblr ||
			shareEmail;
			
	_titleCaptionTemplate = titleCaptionTemplate;
	_folderCaptionTemplate = folderCaptionTemplate;
	
	// Javascript variables compiled upfront
	jsGlobalVars = null;
	jsLightboxVars = null;
	jsSliderVars = null;
	jsMapVars = null;

	// Background audio array
	backgroundMusicFolder = 'res';
	
	relPath = '';
	
	/* 
	 * 	New variables
	 *	All types
	 				fileLabel
	 				commentShort
	 				thumbCaption
	 				shop
	 				
	 *	Folders
	 				folderThumbPath
	 				altText
	 				folderModDate
	 				folderCounts
	 				epochDateRange
	 				
	 *  Displayable objects
	 				imageCaption
	 				mostphotos
	 				epochDate
	 				location
	 				
	 *	Images 
	 				photodata
	 				regions
	 
	 */
			
	var processFolder = function(folder) {
					 
				var	shopCount = new AtomicInteger(0),
					locationCount = new AtomicInteger(0),
					regionCount = new AtomicInteger(0),
					vrCount = new AtomicInteger(0),
					imageNum = new AtomicInteger(0),
					zipCount = new AtomicInteger(0);
					
				//logger(Level.FINE, 'Processing folder "{0}"', folder);
				
				// creating extra variables --> data1.json
				folder.getChildren().forEach(function(ao) { 
					// parallelStream() fails
						
					var vars,
						cat = ao.getCategory(),
						v,
						s,
						t;
						
					if (ao.isIncluded() && 
						!(cat === Category.folder && ao.isHidden()) && 
						(vars = ao.getVars())) {
						
						vars.put('fileLabel', vars.get('fileTitle') || vars.get('label').replace('_', ' '));
											
						if (newDaysRef && newDaysCount) {
							if (cat === Category.folder) {
								switch (newDaysRef) {
									case 'added':
										s = JAlbumUtilities.getDeepLastAdded(ao);
										break;
									case 'dateTaken':
										s = JAlbumUtilities.getDeepCameraDates(ao).last;
										break;
									default:
										s = JAlbumUtilities.deepLastModified(ao);
								}
							} else {
								switch (newDaysRef) {
									case 'added':
										s = 0 + ao.getWhenAdded();
										break;
									case 'dateTaken':
										s = getEpochDate(ao, true);
										break;
									default:
										s = 0 + ao.getLastModified();
								}
							}
							
							if (s) {
								vars.put('modifiedDays', Math.round(s / 86400000));
							}
						}
						
						if (cat === Category.folder || cat === Category.webLocation) {
							
							s = vars.get((cat === Category.folder)? 'description' : 'comment');
							if (preFormat) {
								vars.put('commentShort', shorten(preformatText(s).replace(/(<br>\s?)+$/gi, '')));
							} else {
								vars.put('commentShort', shorten(stripHTML(s)));
							}
							
							if (cat === Category.folder) {
								
								switch (folderDateSource) {
									case 'fileDate':
										vars.put('folderModDate', vars.get('fileDate'));
										break;
										
									case 'folderModDate':
										vars.put('folderModDate', getFormattedDate(deepLastModifiedObject(ao)));
										break;
										
									case 'lastCameraDate':
										var dateRange = JAlbumUtilities.getDeepCameraDates(ao);
										vars.put('folderModDate', getFormattedDate(dateRange.last + 0));
										break;
										
									case 'cameraDateRange':
										var dateRange = JAlbumUtilities.getDeepCameraDates(ao);
										if (dateRange.first && dateRange.last) {
											vars.put('folderModDate', getFormattedDateRange(dateRange.first + 0, dateRange.last + 0));
											break;
										}							
										
									default:
										vars.put('folderModDate', '');
								}
								
								vars.put('counts', getCounts(ao, true, 'p'));
								
							} else {
								vars.put('counts', '');
							}
							
							if (s = vars.get('iconPath')) {
								s = s.replace('folder.png', 'folder.svg');
								vars.put('iconpath', s);
								vars.put('folderThumbPath', s.replace(/^\.\.\//, ''));
							} else {
								vars.put('folderThumbPath', createFolderThumb(ao, folderThumbDims, fixedShapeFolderThumbs)); 
							}
							
							if (_folderCaptionTemplate && (s = processTemplate(ao, _folderCaptionTemplate))) {
								vars.put('thumbCaption', s);
							}
							
						} else {
							
							s = vars.get('comment');
							if (preFormat) {
								s = preformatText(s);
								vars.put('comment', s);
								vars.put('commentShort', shorten(s));
							} else {
								vars.put('commentShort', shorten(stripHTML(s)));
							}
														
							if (cat === Category.webPage) {
								
								t = vars.get('title');
								
								// Page title, HTML tags removed
								vars.put('pageTitle', cleanup(stripHTML(t)));
								
								// Top navigation
								vars.put('topNavigation', _useTopNavigation? getDropdownMenu(rootFolder, ao, !logoName, topNavigationIncludeFolders, topNavigationIncludePages, topNavigationIncludeWebLocations, topNavigationDepth - 1) : '');
								
								if (basePath) {
									vars.put('pageUrl', basePath + relPath + ao.getWebName());
								} else {
									vars.put('pageUrl', '');
								}
								
								// Description
								vars.put('pageDescription', cleanup(stripHTML(s)));
								
								// Folder info: no made date, no image count
								vars.put('folderInfo', '');
								
								// Title as in the Hero
								vars.put('folderTitle', '<h1>' + t + '</h1>' + (s? ('<div class="description">' + s + '</div>') : ''));
								
								// Caption
								vars.put('thumbCaption', '<h6>' + t + '</h6>' + (s? ('<div class="comment">' + s + '</div>') : ''));
						
							} else {
								
								vars.put('imageNum', imageNum.getAndIncrement() + 1);
								
								vars.put('fileSizeFormatted', getSizeAsString(vars.get('fileSize'))); 
													
								if (thumbCaptionTemplate && (s = processTemplate(ao, thumbCaptionTemplate))) {
									vars.put('thumbCaption', s);
								}
						
								if (imageCaptionTemplate && (s = processTemplate(ao, imageCaptionTemplate))) {
									vars.put('imageCaption', s);
								}
								
								if (_usePhotodata && (s = getPhotodata(ao, photoDataTemplate, 'p', showPhotoDataLabel))) {
									vars.put('photodata', s);
								}
								
								if (cat === Category.image) {
									
									if (_useRegions && (s = getRegions(ao)) !== null) {
										vars.put('regions', JSON.stringify(s));
										regionCount.getAndIncrement();
									}
									
									if (s = checkProjectionType(ao)) {
										//vars.put('projectionType', s);
										vrCount.getAndIncrement();
									}
								}
									
								if (_useShop) {
									
									shopCount.getAndIncrement();
									
									if (vars.containsKey('shopOptionsLocal') || 
										vars.containsKey('shopDiscountRateLocal') ||
										vars.containsKey('shopQuantityCapLocal')	) {
								
										var shop = new MyJSONObject();
										if (s = vars.get('shopOptionsLocal')) {
											shop.put('options', s.replace(/\n/g, '::'));
											if (s === '-') {
												shopCount.getAndDecrement();
											}
										}
										if (s = vars.get('shopDiscountRateLocal')) {
											shop.put('discountRate', s);
										}
										if (s = vars.get('shopQuantityCapLocal')) {
											shop.put('quantityCap', s);
										}
										vars.put('shop', shop);
									}
								}
								
								if (_useMap && (s = getLocation(ao))) {
									vars.put('location', s);
									locationCount.getAndIncrement();
								}
								
								if (_useMostPhotos && (s = vars.get('buyImageUrl'))) {
									vars.put('mostphotos', s.replace('http\s?://www.mostphotos.com/', ''));
								}
								
								if (v = getEpochDate(ao)) {
									vars.put('epochDate', v);
								}
								
								if ((v = vars.get('externalContent')) && (v = processTemplate(ao, v, true))) {
									var ext = new MyJSONObject(),
										size = vars.get('externalContentSize');
										
									ext.put('cont', v.trim());
									
									if (!size && (size = getDimFromCode(v.toLowerCase()))) {
										size = size.join('x');
									}
									                              
									if (size) {
										ext.put('size', size);
									}
									
									vars.put('external', ext);
								}
								
								if (zipImages === 'slides' || zipImages === 'originals') {
									zipCount.getAndIncrement();
								} else if (zipImages === 'included' && vars.get('originalPath')) {
									zipCount.getAndIncrement();
								}
							}
						}								
					}
				});
				
				// Current folder variables --> tree.json
				
				relPath = getRelPath(folder);
				
				var	vars;
				
				if (vars = folder.getVars()) {
					
					var c = vars.get('description'),
						t = vars.get('title'),
						modDate,
						topNav,
						top = (folder === rootFolder);
						
					// Counters
					vars.put('shopCount', shopCount.get());
					vars.put('locationCount', locationCount.get());
					vars.put('regionCount', regionCount.get());
					vars.put('vrCount', vrCount.get());
					_anyVr = vrCount.get() > 0;
					
					// Zip file?
					vars.put('zipFile', zipCount.get()? (folder.getWebName() + '.zip') : '');
					
					// Epoch date range
					dateRange = JAlbumUtilities.getDeepCameraDates(folder);
					if (dateRange.first && dateRange.last) {
						vars.put('epochDateRange', (dateRange.first + 0) + ',' + (dateRange.last + 0));
					}
					
					// Folder modified date
					switch (folderDateSource) {
						case 'fileDate':
							vars.put('folderModDate', modDate = vars.get('fileDate'));
							break;
							
						case 'folderModDate':
							vars.put('folderModDate', modDate = getFormattedDate(deepLastModifiedObject(folder)));
							break;
							
						case 'lastCameraDate':
							vars.put('folderModDate', modDate = getFormattedDate(dateRange.last + 0));
							break;
							
						case 'cameraDateRange':
							if (dateRange.first && dateRange.last) {
								vars.put('folderModDate', modDate = getFormattedDateRange(dateRange.first + 0, dateRange.last + 0));
								break;
							}							
							
						default:
							vars.put('folderModDate', modDate = '');
					}
					
					// Neighboring folders
					if (!top && (linkNeighboringFolders || afterLast === 'ask' || afterLast === 'nextfolder')) {
						
						if ((ao = getPreviousFolder(folder)) != null) {
							v = ao.getVars();
							vars.put('previousFolderPath', '../' + v.get('closeupPath'));
							vars.put('previousFolderTitle', ao.getTitle() || ao.getName());
							vars.put('previousFolderThumbPath', '../' + v.get('thumbPath'));
							s = encodeAsJava(ao.getWebName());
							
							if ((ao = getLastImage(ao)) != null) {
								vars.put('previousFoldersLast', '../' + s + '/' + indexName + '#img=' + getFinalName(ao));
							} else {
								vars.put('previousFoldersLast', '');
							}
						} else {
							vars.put('previousFolderPath', '');
							vars.put('previousFolderTitle', '');
							vars.put('previousFolderThumbPath', '');
							vars.put('previousFoldersLast', '');
						}
						
						if ((ao = getNextFolder(folder)) != null) {
							v = ao.getVars();
							vars.put('nextFolderPath', '../' + v.get('closeupPath'));
							vars.put('nextFolderTitle', ao.getTitle() || ao.getName());
							vars.put('nextFolderThumbPath', '../' + v.get('thumbPath'));
							s = encodeAsJava(ao.getWebName());
							
							if ((ao = getFirstImage(ao)) != null) {
								vars.put('nextFoldersFirst', '../' + s + '/' + indexName + '#img=' + getFinalName(ao));
							} else {
								vars.put('nextFoldersFirst', '');
							}
						} else {
							vars.put('nextFolderPath', '');
							vars.put('nextFolderTitle', '');
							vars.put('nextFolderThumbPath', '');
							vars.put('nextFoldersLast', '');
						}
					}
							
					// Page URL and thumb path
					if (basePath) {
						vars.put('pageUrl', basePath + relPath + indexName);
						vars.put('pageThumbPath', basePath + relPath + folderThumbFileName);
					} else {
						vars.put('pageUrl', '');
						vars.put('pageThumbPath', folderThumbFileName);
					}
					
					// Page title, HTML tags removed
					vars.put('pageTitle', cleanup(stripHTML(t)));
					
					// Top navigation
					vars.put('topNavigation', _useTopNavigation? getDropdownMenu(rootFolder, folder, !top && !logoName, topNavigationIncludeFolders, topNavigationIncludePages, topNavigationIncludeWebLocations, topNavigationDepth - 1) : '');
					
					// Bottom navigation
					vars.put('bottomNavigation', showBottomNavigation? getRootNavigation(folder, '') : '');
						
					// Breadcrumb path
					vars.put('breadcrumbPath', getBreadcrumbPath(folder));
					
					// Description
					vars.put('pageDescription', stripQuot(stripHTML(c)));
					
					vars.put('folderInfo', (
							(showModifiedDate && modDate)? 
								('<div class="modifieddate"><span>' + getText('modified') + '</span> ' + modDate + '</div>') 
								: 
								''
						) + (showImageCount? getCounts(folder, true, 'div', 'counts') : ''));
					
					// Title as in the Hero
					if (_titleCaptionTemplate && (s = processTemplate(folder, _titleCaptionTemplate))) {
						vars.put('folderTitle', preFormat? preformatText(s) : s);
					} else {
						vars.put('folderTitle', '<h1>' + t + '</h1><div class="description">' + c + '</div>');
					}
					
					// Processing folder image and thumbnail
					vars.put('hasFolderImage', createFolderImage(folder, folderImageDims));
					
				}
			};
					
		/*
		 *	Initializing Javascript variables
		 */
		 
		getGlobalVars = function() {
				var o = {
							albumName:	stripQuot(albumTitle),
							makeDate:	(new Date()).getTime() / 86400000,
							licensee:	license
						};
						
				o['thumbDims'] = maxThumbWidth + 'x' + maxThumbHeight + (fixedShapeThumbs? '!' : '');
				if (homepageAddress) {
					o['uplink'] = escQuot(homepageAddress);
				}
				if (showCookiePolicy) {
					o['showCookiePolicy'] = true;
					if (cookiePolicyUrl) {
						o['cookiePolicyUrl'] = cookiePolicyUrl;
					}
				}
				
				// Controls
				if (!enableKeyboard) {
					o['enableKeyboard'] = false;
				}
				if (!enableMouseWheel) {
					o['enableMouseWheel'] = false;
				}
				if (rightClickProtect) {
					o['rightClickProtect'] = true;
				}
				if (_anyShares) {
					
					var s = new Array();
					
					if (shareFacebook) s.push('facebook');
					if (shareTwitter) s.push('twitter');
					if (shareGplus) s.push('googlePlus');
					if (shareTumblr) s.push('tumblr');
					if (sharePinterest) s.push('pinterest');
					if (shareLinkedIn) s.push('linkedin');
					if (shareDigg) s.push('digg');
					if (shareStumbleupon) s.push('stumbleupon');
					if (shareReddit) s.push('reddit');
					if (shareEmail) s.push('email');
					o['share'] = { 
						sites:			s.join(','),
						hook:			'.social-links'
					}
					
					s = [];
					if (facebookLike) s.push('facebook');
					if (twitterTweet) s.push('twitter');
					if (googlePlus) s.push('google');
					if (tumblrButton) s.push('tumblr');
					if (pinItButton) s.push('pinterest');
					if (s.length) {
						o['share']['buttons'] = s.join(',');
					}
					
					if (facebookAppId) {
						o['share']['facebookAppId'] = facebookAppId;
					}
					
				}
				if (_useSearch) {
					o['search'] = {
						fields:		searchFields,
						hook:		'.search'
					}
				}
				if (_useFeedback) {
					o['feedback'] = {
						to:				feedbackEmail,
						floatBtnLabel:	feedbackFloatButtonLabel,
						sendBtnLabel:	feedbackSendButtonLabel,
						hook:			'#feedback'
					};
					
					if (feedbackFormatting !== 'human') {
						o['feedback']['formatting'] = feedbackFormatting;
					}
					if (feedbackTemplate) {
						o['feedback']['template'] = feedbackTemplate;
					}
					if (feedbackInstructions) {
						o['feedback']['instructions'] = feedbackInstructions;
					}
				}
				if (extraSizes) {
					o['extraSizes'] = extraSizes;
				}
				if (markFilesNew && newDaysCount) {
					o['markNew'] = newDaysCount;
				}
				/*if (showFolderImageCount) {
					o['showCounts'] = true;
				}*/
				if (thumbCols !== 4) {
					o['thumbGridClass'] = thumbGridClass;
				}
				if (!fixedShapeThumbs || !captionPlacement.equals('tooltip')) {
					o['fixThumbs'] = true;
				}
				if (!fixedShapeThumbs || !folderCaptionPlacement.equals('over')) {
					o['fixFolderThumbs'] = true;
				}
				if (captionPlacement === 'above') {
					o['captionAbove'] = true;
				} else if (captionPlacement === 'tooltip') {
					o['captionTooltip'] = true;
				}
				
				return o;
			},
			
		// Lightbox variables
		
		getLightboxVars = function() {
				var oo = {
							lightbox:	{}
						},
					o = oo.lightbox;
			
				if (afterLast !== 'donothing') {
					o['afterLast'] = afterLast;
				}
				if (transitionType !== 'crossFadeAndSlide') {
					o['transitionType'] = transitionType;
				}
				if (!useThumbnailStrip) {
					o['useThumbstrip'] = false;
				}
				if (!thumbnailsVisible || !useThumbnailStrip) {
					o['thumbsVisible'] = false;
				}
				if (infoPanelVisible == false) {
					o['captionVisible'] = false;
				}
				if (hideFitToggle) {
					o['useZoom'] = false;
				}
				if (hideStartStop) {
					o['useSlideshow'] = false;
				}
				if (fitImages === 'none') {
					o['fitImages'] = false;
				} else if (fitImages === 'vertically') {
					o['fitBoth'] = false;
				}
				if (dontStretchBehind) {
					o['fitBetween'] = true;
				}
				if (!neverScaleUp) {
					o['scaleUp'] = true;
				}
				if (videoAutoPlay) {
					o['videoAuto'] = true;
				}
				if (showImageNumbers) {
					o['showNumbers'] = true;
				}
				if (printImageButton) {
					o['printImage'] = true;
				}
				if (showShare) {
					o['showShare'] = true;
				}
				if (transitionSpeed !== 400) {
					o['speed'] = transitionSpeed;
				}
				if (slideshowDelay != 4) {
					o['slideshowDelay'] = Math.round(slideshowDelay * 1000);
				}
				if (!autohideControls) {
					o['autohideControls'] = false;
				}
				if (slideshowAuto) {
					o['autoStart'] = true;
				}
				if (!use360Player) {
					o['use360Player'] = false;
				}
				if (backgroundAudioSlideshowControl) {
					o['backgroundAudioSlideshowControl'] = true;
				}
				if (!buttonLabelsVisible) {
					o['buttonLabels'] = false;
				}
				if (regionsBtnText && regionsBtnText !== getText('regionsBtn')) {
					o['regionsBtn'] = regionsBtnText;
				}
				if (!clickBesideForIndex) {
					o['quitOnDocClick'] = false;
				}
				if (!clickForNext) {
					o['clickForNext'] = false;
				}
				if (useFullScreen) {
					o['useFullScreen'] = true;
				}
				return oo;	
			},
	
		// Slider vars
		
		/*	
		getSliderVars = function() {
				var o = {};
				
				if (sliderTransition !== 'crossFade') {
					o['transition'] = sliderTransition;
				}
				if (sliderDelay !== 4000) {
					o['delay'] = sliderDelay;
				}
				if (sliderTransitionSpeed !== 600) {
					o['speed'] = sliderTransitionSpeed;
				}
				
				return 	o;
			},
		*/
		
		// Audio player vars
		
		getAudioPlayerVars = function() {
				var o = {};
				
				if (backgroundAudio) {
					o['src'] = backgroundMusic;
					
					if (!backgroundAudioAutoPlay) {
						o['autoPlay'] = false;
					}
					if (!backgroundAudioLoop) {
						o['loop'] = false;
					}
					if (backgroundAudioSlideshowControl) {
						o['slideshowControl'] = true;
					}
					if (!backgroundAudioRetainPosition) {
						o['saveStatus'] = false;
					}
					
					o['folder'] = backgroundMusicFolder;
				}
				
				return o;
			},
			
		// Map vars
		
		getMapVars = function() {
				var o = {
						map: {
							type:	mapType,
							zoom:	mapZoom,
							index:	showMapSection
						}
					};
				
				if (googleApiKey && googleApiKey.charAt(0) !== '#') {
					o.map['apiKey'] = googleApiKey.trim();
				}
				
				return 	o;
			},
	
		// Index page variabes
		
		getIndexVars = function() {
				var o = {
							rootPath:	rootPath,
							resPath:	resPath,
							relPath:	relPath.replace(/\/$/, '')
						};
				
				if (_useTagCloud) {
					o['tagCloud'] = {
						fields:		tagCloudFields.replace(/,\s+/g, ','),
						depth:		tagCloudSource,
						hook:		'#tag-cloud .tag-cloud'
					};
					if (tagCloudSort !== 'none') {
						o['tagCloud']['sort'] = tagCloudSort;
					}
					if (tagCloudFontVaries) {
						o['tagCloud']['fontVaries'] = true;
					}
					if (tagCloudSearch) {
						o['tagCloud']['searchHook'] = '#tag-cloud form';
					}
				}
				
				if (_useSearchNew) {
					o['searchNew'] = {
						days:		searchNewDays.replace(/,\s+/g, ','),
						depth:		searchNewSource,
						hook:		'.search-new'
					}
					if (searchNewReference !== 'added') {
						o['searchNew']['reference'] = searchNewReference;
					}
					if (!searchNewSinceLastVisit) {
						o['searchNew']['sinceLastVisit'] = false;
					}
				}
				
				if (level > 0) {	
					o['level'] = level;
					if (typeof previousFoldersLast !== UNDEF) {
						o['previousFoldersLast'] = previousFoldersLast;
					}
					if (typeof nextFoldersFirst !== UNDEF) {
						o['nextFoldersFirst'] = nextFoldersFirst;
					}
				}
				
				if (folderCols > 2) {
					o['folderCols'] = folderCols;
				}
				
				if (indexName !== 'index.html') {
					o['indexName'] = indexName;
				}
				
				return o;
			},

		//	Get Javascript variable to pass to the skin
		
		getJsVars = function() {
				var v = extend({}, jsGlobalVars, jsMapVars, getAudioPlayerVars(), getIndexVars(), { 'pageType': pageType });
				
				if (pageType === 'index') {
					extend(v, jsLightboxVars);
				}
					
				return (JSON.stringify(v)).replace(/\:true/g, ':!0').replace(/\:false/g, ':!1');
			},
				
	
		/*
		 * Initializing album
		 */
			 
		initAlbum = function() {
			
				// Fixed shape
				if (fixedShapeThumbs) {
					engine.addFilter(new FixedShapeFilter(), JAFilter.THUMBNAILS_PRESCALE_STAGE);
				}
			
				// Uplink
				if (homepageAddress) {
					uplink = homepageAddress;
					uplinkText = homepageLinkText;
				}
				
				if (folderDateSource === 'none') {
					_folderCaptionTemplate = folderCaptionTemplate.replace(/\s*(<span class="date">)?\$\{folderModDate\}<\/span>/g, '');
					_titleCaptionTemplate = titleCaptionTemplate.replace(/\s*(<span class="date">)?\$\{folderModDate\}<\/span>/g, '');
				}
				
				var sb = new Array();
				
				// Custom link
				if (customLink) {
					sb.push('<a href="' + customLink + '" target="_blank">' + 
						((customLinkText)? customLinkText : customLink) +
						'</a>');
				}
				
				// Credits
				if (!engine.isExcludeBacklinks()) {
					// jAlbum credit
					sb.push('<a href="' + generatorUrl + 
						'" rel="generator" data-tooltip title="' + 
						getText('getJalbumNow') + ' (v' + internalVersion + ')">' +
						(creditText? creditText : (getText('credit').replace(/\{0\}/g, getText('photoAlbums')).replace(/\{1\}/g, 'jAlbum'))) +
						'</a>');
					
					// Skin link
					sb.push('<a href="' + skinLink + 
						'" rel="generator" data-tooltip title="' +
						getText('skin') + ': ' + skin + ' ' + styleName + ', ' + 
						skinVersion + '">' + skin + '</a>');
				}
				
				credits = sb.join(' &middot; ');
				
				if (heroPattern) {
					copyResource('patterns/' + (isLightColor(backgroundColor, heroColor)? 'light' : 'dark'), heroPattern);
				}
				
				// MS server configuration
				if (useMsServer) {
					copySkinFile('includes', 'web.config');
				}
		
				// Expiry headers
				if (useExpiry) {
					copySkinFile('includes', '.htaccess');
				}
				
				copySkinFile('includes', 'robots.txt');
				copySkinFile('includes', 'humans.txt');
				
				// Background music
				if (backgroundAudio) {
					backgroundMusic = copyResources(backgroundAudio.split('\t'), backgroundMusicFolder);
					jsAudioPlayerVars = getAudioPlayerVars(); 
				} else {
					backgroundMusic = '';
				}
				
				// JS variables
				jsGlobalVars = getGlobalVars();	
				jsLightboxVars = getLightboxVars();
				jsMapVars = _useMap? getMapVars() : null;
				//jsSliderVars = _useSlider? getSliderVars() : null;
				
				// Creating all.js
				mergeJs('js',
					// Foundation libraries
					'foundation,' + 'what-input,' +
					// Utilities
					'laza.util,' +
					// Album model
					'jalbum.album,' +
					// laza libraries
					'laza.cookie,laza.scrolltop,laza.sharebuttons,laza.transform,laza.swipe,' +
					// Audio player
					(backgroundMusic? 'laza.audioPlayer,' : '') +
					// Map
					((_useMap !== 'none')? 'laza.addmap,' : '') +
					// Lightbox and misc utilities
					'laza.lightbox,laza.alignto,laza.addtooltip,laza.modal,laza.matchheight,' +
					// Paypal
					(_useShop? 'laza.paypal,' : '') +
					// Feedback
					(_useFeedback? 'laza.feedback,' : '') +
					// main Js
					'main',
					// Output name
					'all',
					debugMode,
					[
						// Texts:start
						// modal
						'closeWindow',
						'okButton',
						'warning',
						'error',
						// relative date
						'today',
						'yesterday',
						'daysAgo',
						'monthsAgo',
						'yearsAgo',
						// paypal
						'addCart',
						'shoppingCart',
						'edit',
						'continueShopping',
						'buyNow',
						'processedByPaypal',
						'viewCart',
						'emptyCart',
						'removeAllItems',
						'yes',
						'no',		
						'item',
						'items',
						'success',
						'couponCode',
						'redeem',
						'noSuch',
						'expired',
						'lowerThanCurrent',
						'reclaimed',
						'subtotal',
						'total',
						'shippingAndHandling',
						'reduction',
						'discount',
						'tax',
						'remove',
						'couponAccepted',
						'couponRemoved',
						'amountLowerThan',
						'addMoreItems',
						'validAbove',
						'higherThanTotal',
						'minAmountWarning',
						'minQuantityWarning',
						'maxNItems',
						// feedback
						'continueBrowsing',
						'feedback',
						'feedbackOnAlbum',
						'overLimitWarning',
						// share buttons
						'share',
						'shareOn',
						'checkOutThis',
						'localWarning',
						// search
						'foundNTimes',
						'notFound',
						'search',
						'newImages',
						'label',
						'return',
						'and',
						// search new
						'newItem',
						'today',
						'inTheLastDay',
						'inThePastNDays',
						'inThePastNMonths',
						'inThePastNYears',
						'sinceMyLastVisit',
						'imagesAdded',
						'imagesModified',
						'imagesTaken',
						// lightbox
						'startSlideshow',
						'atFirstPage',
						'atLastPage',
						'atLastPageQuestion', 
						'startOver', 
						'backToHome',
						'stop',
						'pause',
						'upOneLevel',
						'backToIndex',
						'previousPicture',
						'nextPicture',
						'previousFolder',
						'nextFolder',
						'oneToOneSize',
						'fitToScreen',
						'showInfo',
						'hideInfo',
						'showThumbs',
						'hideThumbs',
						'clickToOpen',
						'metaBtn', 
						'metaLabel',
						'mapBtn',
						'mapLabel',
						'shopBtn',
						'shopLabel',
						'shareBtn',
						'shareLabel',
						'download',
						'print',
						'printLabel',
						'fotomotoBtn',
						'fotomotoLabel',
						'mostphotosBtn',
						'mostphotosLabel',
						'regionsBtn',
						'regionsLabel',
						// feedback
						'sendFeedback',
						'message',
						'subject',
						'comment',
						'yourEmail',
						'send',
						'messageSent',
						'errorSending',
						'tooLong',
						'emailMissing',
						'noItemsSelected',
						'selectItemsHint',
						// scroll to top
						'scrollTopTooltip',
						// etc
						'more',
						'less',
						'locationWarning',
						'cookiePolicyText',
						'cookiePolicyAgree',
						'cookiePolicyLearnMore'
						// Texts:end
					]
				);
				
				var vars = rootFolder.getVars();
			
				if (_useShop) {
					var shop = new MyJSONObject();
						
					shop.put('id', shopId);
					shop.put('currency', shopCurrency);
					
					if (shopOptions) {
						shop.put('options', shopOptions.replace(/\n/g, '::'));
					}
					if (shopHandling) {
						shop.put('handling', shopHandling);
					}
					if (shopQuantityCap) {
						shop.put('quantityCap', shopQuantityCap);
					}
					if (shopDiscountRate) {
						shop.put('discount', shopDiscountRate);
					}
					if (shopDiscountMinAmount) {
						shop.put('discountMinAmount', shopDiscountMinAmount);
					}
					if (shopDiscountMinQuantity) {
						shop.put('discountMinQuantity', shopDiscountMinQuantity);
					}
					if (shopCoupons) {
						shop.put('coupons', xEncrypt(shopCoupons));
					}
					if (shopSuccessUrl) {
						shop.put('successUrl', shopSuccessUrl);
					}
					if (shopInstructions) {
						shop.put('instructions', shopInstructions);
					}
					
					vars.put('shop', shop);
				}
							
			};
		
	initAlbum();
	
	
	/*
	print('------------------- inint.js');
	var debugFolder = function(folder) {
		var vars = folder.getVars();
		print(folder.getName() + ' (' + System.identityHashCode(folder) + ')');
		print('[thumbCaption] = ' + (vars? vars.get('thumbCaption') : 'null'));
		for each (ao in folder.getChildren()) {
			if (ao.getCategory() === Category.folder) {
				debugFolder(ao);
			}
		}
	};
	debugFolder(rootFolder);
	print('------------------- index.htt');
	*/
	
