/* 
 *	The Album model
 * 
 *	(c) Laszlo Molnar, 2015, 2017
 *	Licensed under Creative Commons Attribution-NonCommercial-ShareAlike 
 *	<http://creativecommons.org/licenses/by-nc-sa/3.0/>
 *
 *	requires: jQuery, laza.util
 */

 
/*
 * 	Constants
 */
 
;
var J = {
		// jAlbum variables
		ALBUM:			'album',
		FOLDERS: 		'folders',
		NAME: 			'name',
		PATH: 			'path',
		THUMB: 			'thumb',
		IMAGE: 			'image',
		WIDTH: 			'width',
		HEIGHT: 		'height',
		ORIGINAL: 		'original',
		FOLDERS: 		'folders',
		OBJECTS: 		'objects',
		FILEDATE: 		'fileDate',
		COMMENT: 		'comment',
		TITLE: 			'title',
		COUNTERS: 		'counters',
		DEEPCOUNTERS: 	'deepCounters',
		FILESIZE: 		'fileSize',
		CATEGORY: 		'category',
		KEYWORDS: 		'keywords',
		CAMERA: 		'camera',
		VIDEO: 			'video',
		// extra vars
		LEVEL: 			'level',
		PATHREF: 		'pathRef',
		PARENTREF: 		'parentRef',
		RELPATH: 		'relPath',
		IMAGECAPTION: 	'imageCaption',
		THUMBCAPTION: 	'thumbCaption',
		PHOTODATA: 		'photodata',
		LOCATION: 		'location',
		REGIONS:		'regions',
		SHOP:			'shop',
		EXTERNAL:		'external',
		PROJECTIONTYPE:	'projectionType',
		MODIFIEDDAYS: 	'modifiedDays',
		EPOCHDATE:		'epochDate',
		EPOCHDATERANGE:	'epochDateRange',
		MOSTPHOTOS: 	'mostphotos',
		OBJ: 			'obj'
	};

/*
 *	Album object :: use 
 *		myAlbum = new Album({settings});
 *		or
 *		myAlbum = new Album();
 *		myAlbum.init({settings});
 */

var Album = function($, options) {
	
	var instance,
	
		settings = {
				// Name of the tree file
				treeFile: 		'tree.json',
				// Name of the folder data
				dataFile: 		'data1.json',
				// index file name
				indexName: 		'index.html',
				// Name of the slides folder
				slidesDir: 		'slides',
				// Path to root (eg. "../../")
				rootPath: 		'',
				// Default poster images
				audioPoster:	'audio.png',
				videoPoster:	'video.png',
				// Relative path to the current folder (eg. "folder/subfolder") 
				relPath: 		'',
				// Lazy load :: loads folder data only when requested or at the time of initialization
				lazy: 			true,
				// Possible object types
				possibleTypes:	[ 'folder', 'webPage', 'webLocation', 'image', 'video', 'audio', 'other' ]					
			},
			
		// Texts translated
		text = getTranslations({
				and:			'and'
			}),
		
		// Global variables
		// The container for the entire tree
		tree = {},
		// Collection of all album paths in order to be able to store only the references
		paths = [],
		// Path to current folder
		currentFolder,
		// Currently selected item
		current,
		// Collection the JSON promises
		defer = [],
		// Album ready state
		ready,
		
		isReady = function() {
				return ready;
			},
		
		// Returns the whole internal tree object
		
		getTree = function() { 
				return tree; 
			},
			
		// Returns paths array :: debug 
		
		getPaths = function() {
				return paths;
			},
			
		// Filename for images, originalname for other
		
		getItemName = function(o) {
				if (o[J.CATEGORY] === 'video') {
					var p = o[J.VIDEO][J.PATH];
					return decodeURIComponent(p.substring(p.lastIndexOf('/') + 1));
				} else if (o.hasOwnProperty(J.ORIGINAL)) {
					return decodeURIComponent(o[J.ORIGINAL][J.PATH]);
				}
				return o[J.NAME];
			},
			
		// Image?
		
		isImage = function(o) {
				return o.hasOwnProperty(J.CATEGORY) && o[J.CATEGORY] === 'image';
			},
			
		// Audio?
		
		isAudio = function(o) {
				return o.hasOwnProperty(J.CATEGORY) && o[J.CATEGORY] === 'audio';
			},
			
		// Video?
		
		isVideo = function(o) {
				return o.hasOwnProperty(J.CATEGORY) && o[J.CATEGORY] === 'video';
			},
			
		// Folder?
		
		isFolder = function(o) {
				return o.hasOwnProperty(J.LEVEL);
			},
			
		// Ordinary object?
		
		isLightboxable = function(o) {
				return o.hasOwnProperty(J.CATEGORY) && 'image.video.audio.other'.indexOf(o[J.CATEGORY]) !== -1;
			},
	
		// Getting path reference either by searching or adding as new
		
		getPathRef = function(p) {
			
				if (!p) {
					return 0;
				}
			
				if (p.slice(-1) === '/' && p.slice(-3) !== '../') {
					p = p.substring(0, p.length - 1);
				}
				
				var idx = $.inArray(p, paths);
				
				if (idx >= 0) {
					return idx + 1;
				}
				
				return paths.push(p);
			},
		
		// Getting a path by reference index
		
		getPath = function(idx) {
				return idx? paths[idx - 1] : '';
			},
			
		// Gets the path to item
		
		getItemPath = function(o) {
				var p = getPath(o[J.RELPATH]),
					c = o[J.CATEGORY] || 'folder';
				
				p = (p.length && p.slice(-1) !== '/')? (p + '/') : p;	
				
				if (c === 'folder') {
					return p;
				} else if (c === 'video') {
					return p + o[J.VIDEO][J.PATH];
				} else if (c === 'audio' || c === 'other' || o.hasOwnProperty(J.ORIGINAL)) {
					return p + o[J.ORIGINAL][J.PATH];
				} else if (c === 'image') {
					return p + o[J.IMAGE][J.PATH];
				} else if (c === 'webPage') {
					return p + o[J.NAME];
				} else {
					// webLocation
					return o[J.PATH];
				}
			},
			
		// Get the pointer to a folder in a tree from path reference number
		
		getPointer = function(n) {
			
				if (!n) {
					// root
					return tree;
				}
				
				n--;
				
				if (!$.isNumeric(n) || n < 0 || n > paths.length) {
					console.log('Error: out of bounds path reference (' + n + ')!');
					return null;
				}
				
				return getFolder(paths[n]);
			},
				
		// Get the pointer to a folder in a tree from a path
		
		getPointerByName = function(path) {
				
				if (!path) {
					// root
					return tree;
				}
				
				// Removing the closing slash
				if (path[path.length - 1] === '/') {
					path = path.substring(0, path.length - 1);
				}
				
				return getFolder(path);
			},
			
		getFolder = function(path) {
				
				var t = tree,
					p = path.split('/'),
					i, j, k, l;
					
				for (i = 0, k = p.length; i < k; i++) {
					if (t.hasOwnProperty(J.FOLDERS)) {
						for (j = 0, l = t[J.FOLDERS].length; j < l; j++) {
							if (t[J.FOLDERS][j][J.PATH] === p[i]) {
								// found!
								t = t[J.FOLDERS][j];
								break;
							}
						}
						if (j === l) {
							// not found
							return null;
						}
					} else {
						// no subfolders
						return null;
					}
				}
				
				return (i === k)? t : null;
			},
		
		// Getting the parent
		
		getParent = function(o) {
				if (typeof o === UNDEF) {
					o = currentFolder;
				}
				
				if (o === tree) {
					// At top level
					return null;
				}
				
				var p;
				
				if (o.hasOwnProperty(J.PARENTREF)) {
					// folder
					p = getPointer(o[J.PARENTREF]);
				} else {
					// image
					p = getPointer(o[J.PATHREF]);
				}
				
				// Avoid endless loops
				return (p === o)? null : p;
			},
		
		// Retrieving a single property of an element with fallback to upper level folders
		
		getInheritedProperty = function(o, a) {
				if (a.indexOf('.') >= 0) {
					a = a.split('.');
					
					if (a[0] === 'album') {
						return getRootProperty(a[1]);
					}
					
					do {
						if (o.hasOwnProperty(a[0])) {
							return o[a[0]][a[1]];
						}
					} while (o = getParent(o));
					
					return null;
				}
					
				do {
					if (o.hasOwnProperty(a)) {
						return o[a];
					}
				} while (o = getParent(o));
				
				return null;
			},
			
		// Retrieving an Object property of an element with fallback to upper level folders
		
		getInheritedPropertyObject = function(o, a) {
				var p = {};
				do {
					if (o.hasOwnProperty(a)) {
						p = $.extend({}, p, o[a]);
					}
				} while (o = getParent(o));
				
				return p;
			},
			
		// Adding level, parent pointers and relative paths for easier navigation
		
		addExtras = function() {
			
				var add = function(o, level, path) {
						// Level
						o[J.LEVEL] = level;
						
						path = (path.length && path.slice(-1) !== '/')? (path + '/') : path;
						
						var op = level? o[J.PATH] : '';
						
						// path reference
						o[J.PATHREF] = getPathRef(path + op);
						
						// parent ref
						o[J.PARENTREF] = level? getPathRef(path) : null;
						
						// relative path
						var p;
						if (!settings.relPath.length) {
							// root
							p = path + op;
						} else if (o === currentFolder) {
							// current folder
							p = '';
						} else if (path.indexOf(op + '/') === 0) {
							// subfolder
							p = op.substring(settings.relPath.length);
						} else {
							// other branch
							p = settings.rootPath + '/' + (path + op);
						}
						o[J.RELPATH] = getPathRef(p);
						
						if (o.hasOwnProperty(J.FOLDERS)) {
							pr = getPathRef(path);
							for (var i = 0, l = o[J.FOLDERS].length; i < l; i++) {
								add(o[J.FOLDERS][i], level + 1, path + op);
							}
						}
					};
				
				add(tree, 0, '');
			},
		
		// Loading tree.json from the top level folder
		
		loadTree = function(doneFn) {
			
				//console.log('loadTree() :: ' + settings.rootPath + settings.treeFile);
						
				return $.getJSON((settings.rootPath? (settings.rootPath + '/') : '') + settings.treeFile)
					.done(function(d) {
						
						// console.log('... tree loaded'); // Tree loaded
						tree = d;
						// Getting the pointer to the current folder
						currentFolder = getPointerByName(settings.relPath);
						if (currentFolder === null) {
							console.log('Error: can\'t find folder "' + settings.relPath + '" in the database!');
							showError('The current folder\'s database file is missing or broken! ' + 
								(location.protocol === 'file:')?
									'Check if you allowed jAlbum to process the subdirectories and "Make album" again!'
									:
									'If you\'re the owner try to "Upload" the album again!');
							currentFolder = '';
						}
						
						// Adding extra variables
						addExtras();
						
						// Calling "done" function
						if ($.isFunction(doneFn)) {
							doneFn.call(this);
						}
					})
					.fail(function() {
						console.log('Fatal error! Missing or access denied to "' + settings.treeFile + '".');
						if (location.protocol === 'file:') {
							showError('Local access to the album\'s database file is blocked by your browser. This will not affect the uploaded album! Use jAlbum\'s built-in browser or FireFox for testing, or <a href="https://jalbum.net/forum/ann.jspa?annID=172" target="_blank">read here</a> how to test albums in other browsers!');
						} else {
							showError('The album\'s main database file is missing or broken! If you\'re the owner <a href="https://jalbum.net/forum/ann.jspa?annID=177">read how you can fix this</a>.');
						}
						
						// Calling "done" function
						if ($.isFunction(doneFn)) {
							doneFn.call(this);
						}
					});
			},
			
		// Loading a folder's detailed data
		
		loadData = function(folder, doneFn) {
			
				//console.log('loadData("' + f[J.NAME] + '") + ready:' + f[J.READY]);
				var copyFolderProps = function(src, dst) {
						if (!dst) {
							return;
						}
						for (var prop in src) {
							// Assigning folder variables 
							if (prop !== J.OBJECTS && prop !== J.ALBUM && !dst.hasOwnProperty(prop)) {
								dst[prop] = src[prop];
							}
						}
					};
				
				// Couldn't identify/find a folder
				if (!folder) {
					//console.log('Error: loadData("null")!');
					return;
				}	
				
				// Loading the folder's objects
				if (folder.hasOwnProperty(J.OBJECTS)) {
					
					// already loaded
					if ($.isFunction(doneFn)) {
						doneFn.call(this, folder);
					}
					return true;
						
				} else {
					
					var p = getPath(folder[J.RELPATH]);
				
					// we need to load it
					//console.log('Loading "'+ p + '/' + settings.dataFile + '"');
					// building defer array to be able to check the full load
					if (!defer) {
						defer = [];
					}
					
					defer.push($.getJSON( (p? (p + '/') : '') + settings.dataFile).done(function(d) {
						//console.log("data loaded for: " + f[J.NAME]);
						
						// Current folder
						copyFolderProps(d, folder);
						
						// Objects (-folders)
						if (d.hasOwnProperty(J.OBJECTS)) {
							folder[J.OBJECTS] = [];
							for (var i = 0, o, j = 0, l = d[J.OBJECTS].length; i < l; i++) {
								o = d[J.OBJECTS][i];
								if (o[J.CATEGORY] === 'folder') {
									if (!folder[J.FOLDERS]) {
										folder[J.FOLDERS] = [];
									}
									copyFolderProps(o, folder[J.FOLDERS][j++]);
								} else {
									// Not folder
									// Adding absolute and relative paths
									o[J.PATHREF] = folder[J.PATHREF];
									o[J.RELPATH] = folder[J.RELPATH];
									folder[J.OBJECTS].push(o);
								}
							}
						}
						
						if ($.isFunction(doneFn)) {
							doneFn.call(this, folder);
						}
						
					}).fail(function() {
						console.log('Error loading folder data: "' + (p? (p + '/') : '') + settings.dataFile + '".');
						if ($.isFunction(doneFn)) {
							doneFn.call(this, folder);
						}
					}) );
				}
			},
		
		// Loading data for a single folder
		
		loadFolder = function(folder, deep) {
			
				//console.log('loadFolder("' + f[J.NAME] + '")');
				loadData(folder);
				
				if (deep && folder.hasOwnProperty(J.FOLDERS)) {
					
					for (var i = 0, l = folder[J.FOLDERS].length; i < l; i++) {
						loadFolder(folder[J.FOLDERS][i]);
					}
				}
			},
			
		// Initializing
		
		init = function(set) {
			
				if (instance) {
					return instance;
				}
				
				instance = new Date();
				
				if (typeof set !== UNDEF) {
					$.extend(settings, set);
				}
				
				ready = false;
				
				if (settings.rootPath === '.') {
					settings.rootPath = '';
				}
				
				// Loading the folder's Objects: current or all
				
				var treeReady = function() {
					
						defer = [];
						
						loadFolder(settings.lazy? currentFolder : tree, !settings.lazy);
						
						//console.log(defer);
						
						// has subfolders: waiting for AJAX requests to be completed
						$.when.apply($, defer).done(function() {
							
							var d = new Date();
							console.log(defer.length + ' folder(s) loaded: ' + (d - instance) + 'ms');	
							ready = true;
							defer = null;
							current = (currentFolder && currentFolder.hasOwnProperty(J.OBJECTS))? 0 : null;
							
							if ($.isFunction(settings.ready)) {
								//console.log('calling ' + settings.ready);
								settings.ready.call(this);
							}
							
						});
									
					};
				
				// Loading the tree.json first
				
				return loadTree(treeReady);
						
			},
		
		// Album make date/time in UTC
		
		getMakeDate = function() {
				return new Date(tree[J.FILEDATE]);
			},
			
		// Album title
		
		getAlbumTitle = function() {
				return tree[J.TITLE] || tree[J.NAME];
			},
			
		// Current folder object
		
		getCurrentFolder = function() { 
				return currentFolder; 
			},
			
		// Returns all objects in a folder
		
		getObjects = function() {
				return currentFolder.hasOwnProperty(J.OBJECTS)? currentFolder[J.OBJECTS] : []; 
			},
			
		// Returns only the images
		
		getImages = function() {
			
				var items = [];
			
				if (currentFolder && currentFolder.hasOwnProperty(J.OBJECTS)) {
					
					var o = currentFolder[J.OBJECTS];
					
					if (o) {
						for (var i = 0, l = o.length; i < l; i++) {	
							if (isLightboxable(o[i])) {
								items.push(o[i]);
							}
						}
					}
				}
							
				return items;
			},
		
		// Returns only the folders
		
		getFolders = function() {
			
				if (currentFolder && currentFolder.hasOwnProperty(J.FOLDERS)) {
					return currentFolder[J.FOLDERS];
				}
							
				return [];
			},
			
		// Returns the next folder
		
		getNextFolder = function(folder) {
				if (typeof folder === UNDEF) {
					folder = currentFolder;
				}
				
				var f = getParent(folder);
				
				if (f) {
					var i = f[J.FOLDERS].findIndex(function(i) { return i === o; });
					
					if (i < f[J.FOLDERS].length) {
						return f[J.FOLDERS][i + 1];
					}
				}
				
				return null;
			},
		
		// Returns the previous folder
		
		getPreviousFolder = function(folder) {
				if (typeof folder === UNDEF) {
					folder = currentFolder;
				}
				
				var f = getParent(folder);
				
				if (f) {
					var i = f[J.FOLDERS].findIndex(function(i) { return i === o; });
					
					if (i > 0) {
						return f[J.FOLDERS][i - 1];
					}
				}
				
				return null;
			},
				
		// Get next folder's first image
		
		getNextFoldersFirstImage = function(ready) {
				var img,
					folder = getNextFolder(),
					
					getFirstImage = function(folder) {
							if (folder.hasOwnProperty(J.OBJECTS)) {
								for (var o = folder[J.OBJECTS], i = 0, l = o.length; i < l; i++) {
									if (isLightboxable(o[i])) {
										img = o[i];
										return;
									}
								}
							}
						};
				
				if (folder) {
					defer = [];
					loadData(folder, getFirstImage);
					
					if ($.isFunction(ready)) {
						if (defer.length) {
							$.when.apply($, defer).done(function() {
								ready.call(img);
							});
						} else {
							ready.call(img);
						}
					}
				}
				
				return null;
			},
		
		// Get previous folder's last image
		
		getPreviousFoldersLastImage = function(ready) {
				var img,
					folder = getPreviousFolder(),
					
					getLastImage = function(folder) {
							if (folder.hasOwnProperty(J.OBJECTS)) {
								for (var o = folder[J.OBJECTS], i = o.length - 1; i >= 0; i--) {
									if (isLightboxable(o[i])) {
										img = o[i];
										return;
									}
								}
							}
						};
			
				
				if (folder) {
					defer = [];
					loadData(folder, getLastImage);
					
					if ($.isFunction(ready)) {
						if (defer.length) {
							$.when.apply($, defer).done(function() {
								ready.call(img);
							});
						} else {
							ready.call(img);
						}
					}
				}
				
				return null;
			},

		// Gets folder path to an object
		
		getFolderPath = function(o) {
				return getPath(o[J.PATHREF] || 0);
			},
		
		// Gets relative folder path from the current folder
		
		getRelativeFolderPath = function(o) {
				return getPath(o[J.RELPATH] || 0);
			},
			
		// Path to an object as HTML page with hash
		
		getUrl = function(o) {
				o = o || currentFolder;
				var p = getPath(o[J.RELPATH]);
				
				p = (p.length? (p + '/') : '') + settings.indexName;
				
				if (isLightboxable(o[i])) {
					p += '#img=' + encodeAsJave(o[J.NAME]);
				}
				
				return p;
			},
		
		// Thumbnail path
		
		getThumbPath = function(o) {
				var p = getPath(o[J.RELPATH]),
					t = o[J.THUMB][J.PATH];
				
				if (isFolder(o)) {
					t = t.replace(o[J.PATH] + '/', '');
				}
				
				return (p.length? (p + '/') : '') + t;
			},
			
		// Image path
		
		getImagePath = function(o) {
				var p = getPath(o[J.RELPATH]);
				
				p = p.length? (p + '/') : '';
				
				return p + o[J.IMAGE][J.PATH];
			},
			
		// Original path
		
		getOriginalPath = function(o) {
				if (o.hasOwnProperty(J.ORIGINAL)) {
					var p = getPath(o[J.RELPATH]);
					
					p = p.length? (p + '/') : '';
					
					return p + o[J.ORIGINAL][J.PATH];
				}
				
				return null;
			},
			
		// Poster path for audio and video files
		
		getPosterPath = function(o) {
				var p = getPath(o[J.RELPATH]),
					c = o[J.CATEGORY] || 'folder';
				p = p.length? (p + '/') : '';
				
				if ((c === 'audio' || c === 'video') && 
					!o[J.IMAGE][J.PATH].startsWith(settings.slidesDir + '/')) {
					/* custom icon for audio or video */
					return (settings.rootPath.length? (settings.rootPath + '/') : '') + 'res/' + settings[c + 'Poster'];
				}
				return p + o[J.IMAGE][J.PATH];
			},
			
		// Original or source path
		
		getSourcePath = function(o) {
				var p = getPath(o[J.RELPATH]);
				return (p.length? (p + '/') : '') + (o.hasOwnProperty(J.ORIGINAL)? o[J.ORIGINAL][J.PATH] : o[J.IMAGE][J.PATH]);
			},
			
		// Absolute path to an object as HTML page
		
		getAbsolutePath = function(o) {
				var p = getPath(o[J.RELPATH]);
				return (p.length? p.fullUrl() : window.location.href.getDir()) + (window.location.href.getFile() || settings.indexName)  + '#img=' + getItemName(o);
			},
			
		// Absolute image path
		
		getAbsoluteImagePath = function(o) {
				var p = getPath(o[J.RELPATH]);
				return (p.length? p.fullUrl() : window.location.href.getDir()) + o[J.IMAGE][J.PATH];
			},
			
		// Is this the current folder?
		
		isCurrentFolder = function(o) {
				return o[J.RELPATH] === 0;
			},
			
		// Level?
		
		getLevel = function(o) {
				if (typeof o === UNDEF) {
					o = currentFolder;
				}
				return (o === tree)? 0 : o.hasOwnProperty(J.LEVEL)? o[J.LEVEL] : getLevel(getParent(o));
			},
			
		// Title
		
		getTitle = function(o) {	
				if (typeof o === UNDEF) {
					o = currentFolder;
				}
				return o[J.TITLE] || '';
			},
			
		// Name
		
		getName = function(o) {
				if (typeof o === UNDEF) {
					o = currentFolder;
				}
				return o[J.NAME] || '';
			},
		
		// Comment
		
		getComment = function(o) {
				if (typeof o === UNDEF) {
					o = currentFolder;
				}
				return o[J.COMMENT] || '';
			},
			
		
		// Returns a property (normal or inherited way)
		
		getProperty = function(o, a, inherit) {
				if (inherit) {
					return getInheritedProperty(o, a);
				}
				
				if (a.indexOf('.') > 0) {
					a = a.split('.');
					return (o.hasOwnProperty(a[0]))? o[a[0]][a[1]] : null;
				}
				
				return o[a];
			},
		
		// Returns an Object property (normal or inherited way)
		
		getPropertyObject = function(o, a, inherit) {
				return inherit? getInheritedPropertyObject(o, a) : (o.hasOwnProperty(a)? o[a] : null);
			},
		
		// Has shop options?
		
		hasShop = function(o) {
				var p = getInheritedPropertyObject(o || tree, J.SHOP);
				
				return p && p['options'] !== '-';
			},
			
		// Returns a property from the root level
		
		getRootProperty = function(a) {
				return tree.hasOwnProperty(a)? tree[a] : null;
			},

		// Collect items by date
		
		collectByDate = function(options) {
			
				//console.log('collectByDate(' + options + ')');
				if (typeof options === UNDEF || !options.hasOwnProperty('range') || !options.hasOwnProperty('ready')) {
					return;
				}
				
				var options 	= $.extend({
										sort:		true,
										reverse:	false,
										depth: 		'current' 	// 'tree' | 'current' | 'subfolders'
									}, options),
					items 		= [],
					start,
					end, 
				
					
					_findByDate = function(folder) {
							
							// Find images fall into the date range
							
							if (!folder || !folder.hasOwnProperty(J.OBJECTS)) {
								return;
							}
							
							var obj = folder[J.OBJECTS];
							
							for (var i = 0, o; i < obj.length; i++) {
								
								o = obj[i];
								
								if (!isLightboxable(o)) {
									continue;
								}
								
								if (o.hasOwnProperty(J.MODIFIEDDAYS)) {
									// Modified date :: user selected date 
									d = o[J.MODIFIEDDAYS];
								} else if (o.hasOwnProperty(J.EPOCHDATE)) {
									// Fall back to epoch date
									d = o[J.EPOCHDATE] / 86400000;
								} else {
									continue;
								}
								
								if (d >= start && d <= end) {
									items.push(o);
								}
							}
						},
					
					_addFolder = function(folder) {
						
							// Adds one folder
							
							loadData(folder, _findByDate);
							
							if (options.depth !== 'current' && folder.hasOwnProperty(J.FOLDERS)) {
								// recursive to subfolders
								for (var i = 0, l = folder[J.FOLDERS].length; i < l; i++) {
									_addFolder(folder[J.FOLDERS][i]);
								}
							}
						},
						
					_arrangeItems = function() {
							if (options.sort) {
								items.sort(function(a, b) {
									if (b.hasOwnProperty(J.MODIFIEDDAYS)) {
										return options.reverse? 
											a[J.MODIFIEDDAYS] - b[J.MODIFIEDDAYS]
											:
											b[J.MODIFIEDDAYS] - a[J.MODIFIEDDAYS];
									}
									return options.reverse? 
											a[J.EPOCHDATE] - b[J.EPOCHDATE]
											:
											b[J.EPOCHDATE] - a[J.EPOCHDATE];
								});
							}
							if (options.max && options.max < items.length) {
								items = items.slice(0, options.max);
							}
						};
				
				if (options.end) {
					// Absolute
					end = options.end;
					start = end - options.range;
				} else {
					// Relative
					start = Math.ceil((new Date()) / 86400000);
					start -= (options.hasOwnProperty('start'))? options.start : options.range;
					end = start + options.range;
				}
				
				// Adding folder(s)
				defer = [];
				_addFolder((options.depth === 'tree')? tree : currentFolder);
				
				if ($.isFunction(options.ready)) {
					if (defer.length) {
						$.when.apply($, defer).done(function() {
							_arrangeItems();
							options.ready.call(items, options);
						});
					} else {
						_arrangeItems();
						options.ready.call(items, options);
					}
				}
			},
		
		/*
		 *	Collecting search results
		 *
		 *	fields: 	fields to watch
		 *	types:		all or comma separated list of allowed types ('image|audio|video|...)
		 *	depth:		where to collect ('tree' | 'current' | 'subfolders')
		 *	exact:		exact search (or conjunctive)
		 *	max:		maximum number of results
		 */
		
		collectItems = function(options) {
			
				//console.log('collectItems(' + set + ')');
				if (typeof options === UNDEF || !options.hasOwnProperty('terms')) {
					return;
				}
				
				var options 		= $.extend({
											fields: 	'creator,keywords,title,comment,name',
											types:		'all',
											depth: 		'current', 		// 'tree' | 'current' | 'subfolders'
											exact: 		false
										}, options),
					items 			= [], 
					fields 			= options.fields.split(/,\s?/), 
					fieldslength 	= fields.length,
					exact			= new Array(fieldslength),
					terms,
					termslength,
					conjunctive 	= false,
					allTypes		= options.types === 'all',
					types 			= {},
										
					
					_searchItem = function(o, cat) {
						
							var found = conjunctive? new Array(termslength) : null;
								
								matchKey = function(o, v, ex) {
										if (typeof v === 'string') {
											for (var k = 0; k < termslength; k++) {
												if ((ex && terms[k].testMatch(v)) || 
													(!ex && terms[k].testIn(v))) {
													// Found a match
													if (conjunctive) {
														found[k] = true;
													} else {
														items.push(o);
														return true;
													}
												}
											}
										}
										
										return false;
									};
							
							for (var i = 0, j, f, r; i < fieldslength; i++) {
								
								if (fields[i].length > 1) {
									if (fields[i][0] !== cat) {
										continue;
									}
									f = fields[i][1];
								} else {
									f = fields[i][0];
								}
								
								if (o.hasOwnProperty(f)) {
									if ($.isArray(o[f])) {
										for (j = 0; j < o[f].length; j++) {
											r = matchKey(o, o[f][j], exact[f]);
											if (!conjunctive && r) {
												break;
											}
										}
									} else {
										r = matchKey(o, o[f], exact[f]);
										if (!conjunctive && r) {
											break;
										}
									}
								}
							}
							
							
							if (conjunctive && allTrue(found)) {
								// all terms found
								items.push(o);
							}
						},
						
					_searchFolder = function(folder) {
						
							if (!folder) {
								return;
							}
							
							if (folder !== tree && (allTypes || types['folder'])) {
								// Folders but not the top level
								_searchItem(folder, 'folder');
							}
							
							if (folder.hasOwnProperty(J.OBJECTS)) {
								// Objects
								for (var i = 0, o = folder[J.OBJECTS]; i < o.length; i++) {
									cat = o[i][J.CATEGORY];
									if (allTypes || types[cat]) {
										_searchItem(o[i], cat);
									}
								}
							}
						},
				
					_addFolder = function(f) {
						
							// Adds one folder
							
							loadData(f, _searchFolder);
							
							if (options.depth !== 'current' && f.hasOwnProperty(J.FOLDERS)) {
								// recursive to subfolders
								for (var i = 0, l = f[J.FOLDERS].length; i < l; i++) {
									_addFolder(f[J.FOLDERS][i]);
								}
							}
						},
						
					_arrangeItems = function() {
							if (options.max && options.max < items.length) {
								items = items.slice(0, options.max);
							}
						};
				
				// Exact search with quotes: "something exact"
				if (options.terms[0] === '"' && options.terms[options.terms.length - 1] === '"') {
					terms = options.terms.substring(1, options.terms.length - 1);
					if (options.exact === false) {
						// Change only if no excplicit exact spec.  
						options.exact = true;
					}
				} else {
					if (options.exact === false) {
						terms = options.terms.replace(/\s+/g, ",");
						// Preparing conjunctive search
						if (~terms.indexOf(',' + text.and + ',')) {
							terms = terms.replace(new RegExp(',' + text.and + ',', 'gi'), ',');
							conjunctive = true;
						}
							
					} else {
						terms = options.terms.trim();
					}
				}	
				
				terms = options.exact? [ terms ] : removeEmpty(terms.split(/,\s?/));
				termslength = terms.length;
				
				for (var i = 0, f; i < fieldslength; i++) {
					fields[i] = fields[i].split(':');
					f = fields[i][1] || fields[i][0];
					exact[f] = (typeof options.exact === 'string')? (options.exact.indexOf(f) >= 0) : options.exact;
				}
				
				if (!allTypes) {
					
					if (settings.types.charAt(0) === '-') {
						// Negative
						for (var i = 0; i < settings.possibleTypes.length; i++) {
							if (settings.types.indexOf(settings.possibleTypes[i]) === -1) {
								types[settings.possibleTypes[i]] = true;
							}
						}
					} else {
						// Positive
						for (var i = 0, t = settings.types.split(/,\s?/); i < t.length; i++) {
							types[t[i]] = true;
						}
					}
				}
				
				// Starting a new promise collect
				
				defer = [];
				
				// Adding folder(s)
				
				_addFolder((options.depth === 'tree')? tree : currentFolder);
				
				if ($.isFunction(options.ready)) {
					if (defer.length) {
						$.when.apply($, defer).done(function() {
							_arrangeItems();
							options.ready.call(items, options);
						});
					} else {
						_arrangeItems();
						options.ready.call(items, options);
					}
				}
			},
		
		// Tag cloud
		
		collectTags = function(options) {
			
				//console.log('collectTags(' + set + ')');
					
				var options 		= $.extend({
											fields: 	'creator,keywords,folder:title,webLocation:title',
											types:		'all',	
											depth: 		'current', 			// 'tree' | 'current' | 'subfolders'
											exact:		'creator,keywords,name'
										}, options),
					tags 			= [], 
					fields 			= $.isArray(options.fields)? options.fields : options.fields.split(/,\s?/), 
					fieldslength 	= fields.length,
					sortByName 		= options.sort === 'name',
					allTypes 		= options.types === 'all',
					types			= {},
					exact			= {},
					
					_addKey = function(key, type) {
							
							if (!key) {
								return;
							}
							
							var k, 
								ka;
								
							if (exact[type]) {
								ka = [ key.toString() ];
							} else {
								if (type === 'comment') {
									key = key.stripHTML();
								}
								ka = key.split(/[\s,\.\!\?\(\)]+/);
								ka = removeEmpty(ka);
							}
							
							for (var i = 0, l = ka.length; i < l; i++) {
							
								k = ka[i];
								
								if (k.length <= 2) {
									// Empty or too short
									continue;
								}
								// Adds one key or increments the counter if already added
								//console.log('addKey(' + k + '[' + (typeof k) + '])');
								
								for (var t = 0, tl = tags.length; t < tl; t++) {
									//console.log('[' + t + '] ' + tags[t][0] + '[' + typeof(tags[t][0])  + '] (' + ((tags[t][0] == k)? 'found' : 'nope') + ')');
									if (tags[t][0] === k) {
										break;
									}
								}
								
								if (t === tl) {
									// New tag
									tags.push([k, 1]);
								} else {
									// Increment
									tags[t][1]++;
								}
							}
						},
				
					_getKeys = function(o, cat) {
							for (var i = 0, f, keys = ''; i < fieldslength; i++) {
								if (fields[i].length > 1) {
									if (fields[i][0] !== cat) {
										continue;
									}
									f = fields[i][1];
								} else {
									f = fields[i][0];
								}
								
								if (o.hasOwnProperty(f) && o[f]) {
									if ($.isArray(o[f])) {
										for (var j = 0; j < o[f].length; j++) {
											if (!keys || keys.indexOf(o[f][j] + '@') === -1) {
												// Avoid duplicate finds
												_addKey(o[f][j], f);
												keys += o[f][j] + '@';
											}
										}
									} else if (!keys || keys.indexOf(o[f] + '@') === -1) {
										// Avoid duplicate finds
										_addKey(o[f], f);
										keys += o[f] + '@';
									}
								}
							}
						},
						
					_addFields = function(folder) {
					
							// Adds fields from objects array
							if (!folder) {
								return;
							}
							
							if (allTypes || types['folder']) {
								// Current folder
								_getKeys(folder, 'folder');
							}
							
							if (folder.hasOwnProperty(J.OBJECTS)) {
								// Ordinary objects
								for (var i = 0, o = folder[J.OBJECTS], cat; i < o.length; i++) {
									cat = o[i][J.CATEGORY];
									if (allTypes || types[cat]) {
										_getKeys(o[i], cat);
									}
								}
							}
						},
				
					_addFolder = function(folder) {
							
							// Adds one folder
							
							loadData(folder, _addFields);
							
							if (options.depth !== 'current' && folder.hasOwnProperty(J.FOLDERS)) {
								// recursive to subfolders
								for (var i = 0, l = folder[J.FOLDERS].length; i < l; i++) {
									_addFolder(folder[J.FOLDERS][i]);
								}
							}
						},
						
					_arrangeTags = function() {
							if (options.sort) {
								tags.sort(function(a, b) {
									return sortByName? ('' + a[0]).localeCompare('' + b[0]) : (b[1] - a[1]);	
								});
							}
							if (options.max && options.max < tags.length) {
								tags = tags.slice(0, options.max);
							}
						};
				
				// Starting a new promise collect
				
				defer = [];
				
				for (var i = 0, f; i < fieldslength; i++) {
					fields[i] = fields[i].split(':');
					f = fields[i][1] || fields[i][0];
					exact[f] = (typeof options.exact === 'string')? (options.exact.indexOf(f) >= 0) : options.exact;
				}
				
				if (!allTypes) {
					for (var i = 0, t = settings.types.split(/,\s?/); i < t.length; i++) {
						types[t[i]] = true;
					}
				}
				// Adding folder(s)
				
				_addFolder((options.depth === 'tree')? tree : currentFolder);
				
					
				if ($.isFunction(options.ready)) {
				
					if (defer.length) {
						$.when.apply($, defer).done(function() {
							_arrangeTags();
							options.ready.call(tags, options);
						});
					} else {
						_arrangeTags();
						options.ready.call(tags, options);
					}
				}
						
			},
			
		processTemplate = function(template, item) {
			
				var item = item || currentFolder,
					m,
					v;
				
				if (template && template.indexOf('${') > 0) {
				
					while (m = template.match(/\$\{([\w\.]+)\}/)) {
						if (m[1]) {
							v = getProperty(item, m[1], true) || '';
						}
						template = template.substring(0, m.index) + v + template.substring(m.index + m[0].length);
					}
				}
		
				return template;
			},
			
		showError = function(err) {
			
				var el = ($('<div>')
							.css({
								position:			'fixed',
								width:				'80%',
								maxWidth:			'600px',
								top:				'50%',
								left:				'50%',
								transform:			'translate(-50%, -50%)',
								textAlign:			'center',
								padding:			'1em',
								backgroundColor:	'#a00',
								color:				'#e8e8e8'
							})
							.append($('<h5>', {
									text:			'Error'
								}).css({
									color:			'#f63'
								}))
							.append($('<p>', {
									html:			err
								}).css({
									marginBottom:	0
								}))
						).appendTo($('body'));
						
				el.find('a').css({
						color: 				'#fff',
						textDecoration:		'underline'
					});
				
				setTimeout(function() {
						el.fadeOut(function() {
								el.remove();
							});
					}, 6000);
			};
	
	//console.log("Album initialized!");
	
	if (options) {
		if (DEBUG) {
			console.log('new Album(' + JSON.stringify(options) + ');');
		}
		init(options);
	}
	
	return {
			//init: 							init,
			isReady:						isReady,
			// Search
			collectTags: 					collectTags,
			collectItems: 					collectItems,
			collectByDate: 					collectByDate,
			// Debug
			getTree: 						getTree,
			getPaths: 						getPaths,
			// Type checking
			isImage: 						isImage,
			isAudio: 						isAudio,
			isVideo: 						isVideo,
			isLightboxable: 				isLightboxable,
			isCurrentFolder: 				isCurrentFolder,
			// Acceess
			getLevel: 						getLevel,
			getTitle: 						getTitle,
			getName: 						getName,
			getComment: 					getComment,
			getMakeDate: 					getMakeDate,
			getAlbumTitle: 					getAlbumTitle,
			getCurrentFolder: 				getCurrentFolder,
			getObjects: 					getObjects,
			getImages: 						getImages,	
			getFolders: 					getFolders,	
			getFolderPath: 					getFolderPath,
			getRelativeFolderPath: 			getRelativeFolderPath,
			getParent: 						getParent,
			getItemName:					getItemName,
			getItemPath: 					getItemPath,
			getOriginalPath:				getOriginalPath,
			getUrl: 						getUrl,
			getThumbPath: 					getThumbPath,
			getImagePath: 					getImagePath,
			getPosterPath: 					getPosterPath,
			getSourcePath: 					getSourcePath,
			getAbsolutePath: 				getAbsolutePath,
			getAbsoluteImagePath: 			getAbsoluteImagePath,
			getPreviousFoldersLastImage: 	getPreviousFoldersLastImage,
			getNextFoldersFirstImage: 		getNextFoldersFirstImage,
			getProperty: 					getProperty,
			getPropertyObject: 				getPropertyObject,
			hasShop: 						hasShop,
			getRootProperty: 				getRootProperty,
			processTemplate:				processTemplate
				
		};
		
};