/*	
 *	feedback() :: setting up the shopping cart with Feedback
 *
 *	Usage: $(targetElement).feedback( [items,] options );
 *
 *	Dual licensed under the MIT and GPL licenses.
 *	- http://www.opensource.org/licenses/mit-license.php
 *	- http://www.gnu.org/copyleft/gpl.html
 *
 *	Items: jQuery element (array) holding the elements
 *
 *	Options:
		to: 			(the email address),
		cc:				(cc email),
		subject:		(default subject),
		template: 		(template),
*/

;(function($, $window, $body) {
	'use strict';
	
	$.fn.feedback = function(album, settings) {
		
		if (!album) {
			return this;
		}
		
		settings = $.extend({}, $.fn.feedback.defaults, album.getRootProperty(J.FEEDBACK), settings);
		
		if (!settings.to) {
			console.log('feedback.js: no target email found');
			return this;
		}
		
		var // global vars
			self = 		$(this).eq(0),					// parent element					
			ns = 		self.data('lfb_ns'),			// event namespace
			text = 		$.fn.feedback.text,				// texts
			to = 		settings.to.replace('|', '@'),	// send target
			itemTemplate,								// Item template
			
			LBR =		'%0D%0A',						// line break
			TAB =		'%09',							// Tab
			SPACE = 	'%20',							// Space
			AMP =		'%26',							// Ampersand
			
			// jQuery elements:
			cart,										// the cart parent
			popup,										// modal window
			cont,										// modal window content
			summary,									// short list
			details,									// long list
			newItems,									// newly added items
			sendBtn,									// 'Send' button
			emptyCartBtn,								// 'Empty' button
			floatBtn,									// floating shortcut button
						
			id = {
					cart:			'feedback-cart',
					shortcut:		'feedback-shortcut',
					window:			'feedback-window',
					cont:			'cont',
					beforeItems:	'before-items',
					summary:		'summary',
					summaryTxt:		'summary-txt',
					details:		'details',
					newItems:		'new-items',
					items:			'items',
					afterItems:		'after-items',
					buttons:		'buttons',
					item:			'item',
					path:			'path',
					file:			'file',
					info:			'info',
					thumb:			'thumb',
					title:			'title',
					data:			'data',
					options:		'options'
				},
			
			cookie = {
					data:			'feedback.cart.data',
					date:			'feedback.cart.date'
				},
			
			// Converts data from HTML form element into Object (used in saveData)
			
			getFormData = function(f) {
					return f;
				},
			
			wipeData = function() {
					if (LOCALSTORAGE) {
						for (var c in cookie) {
							localStorage.removeItem(cookie[c]);     
						}
					}
				},
			
			// Saving the cart to the localStorage
			
			saveData = function(doneFn) {
				
					if (false && LOCALSTORAGE) {
						var data = [],
							o;
						
						cart.find('.'+id.item).each(function() {
							o = getFormData($(this));
							if (!isEmpty(o)) {
								data.push(o);
							}
						});
						
						if (data.length) {
							data = JSON.stringify(data);
							localStorage.setItem(cookie.data, data);
							// Saving current time
							localStorage.setItem(cookie.date, (new Date()).getTime());
						} else {
							wipeData();
						}
					}
					
					if ($.isFunction(doneFn)) {
						doneFn.call(this);
					}
				},
			
			// Loading the cart from the localStorage
			
			loadData = function(doneFn) {
				
					if (false && LOCALSTORAGE) {
						var data,
							date = (new Date()).getTime(),
							d;
							
						if (d = localStorage.getItem(cookie.date)) {
							if ((date - parseInt(d)) > settings.expiry) {
								wipeData();
								return;
							} else {
								localStorage.setItem(cookie.date, date);
							}
						}
							
						if (d = localStorage.getItem(cookie.data)) {
							
							var	i;
							
							data = JSON.parse(d);
							
							for (i = 0; i < data.length; i++) {
								addItem(details, data[i]);
							}
						}
					}
					
					updateCart(true);
					
					if ($.isFunction(doneFn)) {
						doneFn.call(this);
					}
				},
						
			
			// Adding one item
			
			addItem = function(target, item) {
				
					var path = album.getFolderPath(item);
					
					// Creating the item
					var	it,
						lb;
	
					it = $('<div>', {
							'class':	id.item + ' clearfix'
						});
					
					lb = $('<div>', {
							'class':	id.info
						}).appendTo(it);
						
					// Remove button
					lb.append($('<a>', {
							'class': 	'button icon-cancel'
						}).on('click.' + ns, function() {
							$(this).parents('.' + id.item).remove();
						}).on('selectstart', function(e) {
							e.preventDefault();
							return false;
						}));
						
					// Thumb image
					lb.append($('<img>', {
							'class': 	id.thumb,
							src: 		(path? getRelativePath(settings.relPath, path) : (settings.rootPath + '/')) + settings.thumbsFolder + item[J.NAME]
						}));
					
					// Title
					lb.append($('<span>', {
							'class': 	id.title,
							html: 		item[J.TITLE] || item[J.NAME] || ''
						}));
					
					// Data from template
					it.append($('<div>', {
							'class': 	id.data,
							html:		album.processTemplate(itemTemplate, item)
						}));
						
					target.append(it);
					
					it.data('item', item);
				},
			
			// Converts jAlbum.Album object into Object
			
			getItem = function(item) {
					
					return {
							title: 		item[J.TITLE] || item[J.NAME] || '',
							path:		album.getFolderPath(item),
							file:		item[J.NAME]
						};
				},
			
			// adding jAlbum.Album objects
			
			addItems = function(items) {
					
					if (!items || !items.length) {
						return;
					}
					
					if ($.isArray(items)) {
						if (items.length > 1) {
							moveNewItems();
						}
						for (var i = 0; i < items.length; i++) {
							if (addItem(newItems, items[i]) === -1) {
								break;
							}
						};
					} else {
						addItem(newItems, items);
					}
					
					updateCart();
					
					showCart();
				},
			
			// get the number of items
			
			getItemCount = function() {
					return cont.find('.' + id.item).length
				},
				
			// updating the summary row
			
			updateSummary = function() {
					var items = details.find('.' + id.item),
						stxt = summary.find('.' + id.summaryTxt),
						l = items.length;
						
					stxt.empty();
					
					if (!l) {
						summary.hide();
					} else {
						for (var i = 0; i < 3; i++) {
							if (i >= l) {
								break;
							}
							stxt.append($('<img>', {
									src: 	items.eq(i).find('img.' + id.thumb).attr('src')
								}));
						}
						
						if (l > 3) {
							stxt.append(' +' + (l - 3) + ' ' + (text[(l > 4)? 'items' : 'item']));
						}
						
						details.hide();
						summary.show();
					}
				},		
				
			// Showing the details (instead of summary)
			
			showDetails = function() {
					if (details.is(':hidden')) {
						moveNewItems();
						summary.hide();
						details.fadeIn(400);
					}
				},
			
			// Number of items
			
			itemCount = function() {
					return cont.find('.' + id.item).length;
				},
					
			
			// Moving new items into the details from newItems
			
			moveNewItems = function() {
				
					newItems.find('.' + id.item).each(function() {
						$(this).appendTo(details);
					});
					
					newItems.empty().hide();
					
					updateSummary();
				},
			
			// Something has Changed
			
			updateCart = function(initial) {
					var cnt = itemCount();
					
					newItems.toggle(!newItems.is(':empty'));
					summary.toggle(!(details.is(':empty') || details.is(':visible')));
					sendBtn.add(emptyCartBtn).toggleClass('disabled', cnt === 0);
					
					updateFloatBtn(cnt);
					
					if (!cart.is(':visible') && cnt) {
						floatBtn.fadeIn(400);
					}
					
					if (!initial) {
						saveData();
					}
				},
			
			// Updating the float button

			updateFloatBtn = function(c) {
					c = (typeof c === UNDEF)? itemCount() : c;
					floatBtn.find('.button').text(' ' + settings.floatBtnLabel + ((c > 0)? (' (' + c + ')') : ''));
				},
						
			// Sending the cart
			
			sendCart = function() {
				
					var message,
						head = 	'mailto:' + encodeURIComponent(to) +
								'?subject=' + encodeURIComponent(popup.find('header').text()),
						body = 	[],
						f,
						
						readForm = function(f) {
								if (f) {
									var v,
										b = [];
										
									f.find('input,textarea,select').each(function() {
											if ((v = $(this).val()) !== null) {
												b.push((this.name || this.id || this.nodeName) + ': ' + v);
											}
										});
									
									return b.join(', ');
								}
								
								return '';
							},
							
						format = function(f) {
								if (f) {
									switch (settings.formatting) {
										case 'human':
											return f.serialize().replace(/\+/g, SPACE).replace(/=/g, ': ').replace(/\&/g, LBR);
										case 'serialized':
											return (f.attr('name') || f.attr('id') || 'form') + '="' + encodeURIComponent(f.serialize()) + '";';
										default:
											return encodeURIComponent(readForm(f));
									}
								}
								
								return '';
							};
						
					if ((f = cont.find('.' + id.beforeItems + ' form')).length) {
						body.push(format(f));
					}
					
					if (settings.formatting === 'serialized') {
						body.push('items={');
					} else {
						body.push(LBR + '----------------------');
					}
					
					cont.find('.' + id.item).each(function(i) {
						var item = $(this).data('item'),
							form = $(this).find('form').eq(0);
						
						switch (settings.formatting) {
							
							case 'human':
								body.push(
										(i? LBR : '') + (String)(i + 1) + '.' + TAB + 
										[album.getFolderPath(item), album.getName(item)].join('/') + 
										(form? (LBR + TAB + form.serialize().replace(/\+/g, SPACE).replace(/=/g, ': ').replace(/\&/g, ',' + SPACE)) : '') 
									);
								break;
								
							case 'serialized':
								body.push(
										'{file:"' + [album.getFolderPath(item), album.getName(item)].join('/') + '"' +
										(form? (',' + (f.attr('name') || f.attr('id') || 'form') + ':"' + encodeURIComponent($(this).find('form').serialize()) + '"') : '') + 
										'},'
									);
								break;
								
							default:
								body.push(
										(String)(i + 1) + '.' + TAB + 
										[album.getFolderPath(item), album.getName(item)].join('/') + 
										(form? (':' + TAB + encodeURIComponent(readForm(form))) : '') 
									);
								break;
						}
					});
					
					if (settings.formatting === 'serialized') {
						body[body.length - 1] = body[body.length - 1].replace(/\,$/, '');
						body.push('};');
					} else {
						body.push('----------------------' + LBR);
					}
					
					if ((f = cont.find('.' + id.afterItems + ' form')).length) {
						body.push(format(f));
					}
					
					if (DEBUG) {
						console.log(decodeURIComponent(body.join('\n')));
					}
					
					message = head + '&body=' + body.join(LBR);
						
					if (message.length > 2048) {
						$body.modal($('<div>', {
								html: 	'<p class="icon-warning">' + text.overLimitWarning + '</p>' + '<p><textarea rows="8">' + decodeURIComponent(body.join('\n')) + '</textarea></p>'
							}), {
								'class': 	'alert'
							});
						message = head;
					}
					
					window.location.href = message;
					
					cont.find('.item').remove();
					
					updateCart();
					hideCart();
					
				},

			// Creating the cart
			
			init = function(doneFn) {
				
					var cnt,
						btns;
						
					// Cart exists? Yes: clean old event handlers and div's
					if (ns !== undefined) {
						$(popup).off('.' + ns);
						$('#cart_' + ns).remove();
						$('#cart_shortcut_' + ns).remove();
					}
					
					// Creating new namespace
					self.data('lfb_ns', ns = 'lfb_' + Math.floor(Math.random() * 10000));
					
					// Creating the modal base (full screen)
					cart = $('<div>', {
							id: 		'cart_' + ns,
							'class': 	'modal ' + id.cart,
							'role':		'modal'
						}).hide().appendTo($body);
					
					// Creating the floating cart show button
					if (settings.useFloatButton) {
						floatBtn = $('<div>', {
								id:			'cart_shortcut_' + ns,
								'class': 	id.shortcut
							})
							.hide()
							.append($('<a>', {
									'class':	'small button icon-email'
								})
								.on('click.' + ns, showCart))
							.appendTo(self);
					}
					
					// The inside popup
					popup = $('<div>', {
							'class': 	'window has-header ' + id.window,
							role: 		'dialog'
						}).appendTo(cart);
					
					// Header
					popup.append($('<header>', {
							'class': 	'icon-email',
							html: 		'<strong> ' + text.feedbackOnAlbum.template(album.getAlbumTitle()) + '</strong>' 
						}));
					
					// Close button
					popup.append($('<a>', {
							'class': 	'btn close',
						}).on('click.' + ns, hideCart));
					
					cart.on('click.' + ns, function(e) {
							if (e.target.id === 'cart_' + ns) {
								hideCart();
								return false;
							};
							return true;
						});
					
					// Modal window content
					cont = $('<div>', {
							'class': 	'content'
						}).appendTo(popup);
					
					cnt = settings.template.split(settings.itemsStart)[0].trim();
					
					if (cnt) {
						cont.append($('<div>', {
								'class':	id.beforeItems,
								html:		album.processTemplate(cnt)
							}));
					}
					
					// Summary row
					summary = $('<div>', {
							'class': 	id.summary + ' clearfix'
						}).hide().appendTo(cont);
					
					summary.append($('<div>', {
							'class': 	id.summaryTxt
						}));
					
					// Show details button
					summary.append($('<a>', {
							'class': 	'details-btn icon-drop-down',
							text: 		' ' + text.edit
						}).on('click.' + ns, showDetails));
					
					// Details container
					details = $('<div>', {
							'class': 	id.details + ' ' + id.items
						}).hide().appendTo(cont);
					
					// Container for recently added items
					newItems = $('<div>', {
							'class': 	id.newItems + ' ' + id.items
						}).appendTo(cont);
						
					cnt = settings.template.split(settings.itemsStop);
					
					if (cnt.length && (cnt = cnt[1].trim())) {
						cont.append($('<div>', {
								'class':	id.afterItems,
								html:		album.processTemplate(cnt)
							}));
					}
									
					// Buttons
					btns = $('<div>', {
							'class': 	id.buttons
						}).appendTo(cont);
					
					btns.append($('<a>', {
							'class': 	'secondary button icon-arrow-left',
							text: 		' ' + text.continueBrowsing
						}).on('click.' + ns, hideCart));
					
					emptyCartBtn = $('<a>', {
							'class': 	'alert button icon-trash',
							text: 		' ' + text.emptyCart
						}).on('click.' + ns, emptyCart).appendTo(btns);
					
					sendBtn = $('<a>', {
							'class': 	'disabled button icon-email',
							html: 		' ' + settings.sendBtnLabel
						}).on('click.' + ns, sendCart).appendTo(btns);
										
					// Instructions
					if (settings.hasOwnProperty('instructions') && settings.instructions) {
						cont.append($('<div>', {
								'class': 	'instructions icon-info'
							})
							.append($('<div>')
							.append(settings.instructions.fixjAlbumPaths(settings.resPath, settings.rootPath, settings.relPath))));
					}
					
					// Adding tooltip texts
					//floatBtn.find('a.button').addTooltip(text.viewCart);
					popup.find('a.close').addTooltip(text.continueBrowsing);
					
					if (settings.template.indexOf(settings.itemsStart) >= 0) {
						cnt = settings.template.split(settings.itemsStart)[1].trim();
						if (cnt) {
							cnt = cnt.split(settings.itemsStop)[0].trim();
							if (cnt) {
								itemTemplate = cnt.replace(/\$\{fileName\}/g, '\$\{name\}');
							}
						}
					}
						
					if ($.isFunction(doneFn)) {
						doneFn.call(this);
					}
				},
			
			emptyCart = function() {
					$body.modal($('<h3>', {
							'class':	'text-center icon-warning',	
							html:		' ' + text.removeAllItems
						}), [{
							t: 	text.yes,
							c:	'alert icon-trash',
							h: 	function() {
									cont.find('.item').remove();
									updateCart();
									hideCart();
								}
						}, {	
							t: 	text.no,
							h:	function() {}
						}], {
							'class':	'alert small'
						});
				},
			
			showCart = function(doneFn) {			
					if (!cart.is(':visible')) {
						$body.addClass('no-scroll');
						$body.addClass('has-modal');
						cart.fadeIn(400, function() {
							if ($.isFunction(doneFn)) {
								doneFn.call(this);
							}
						})
					}
					floatBtn.fadeOut();
				},
			
			hideCart = function(doneFn) {
					var c = getItemCount();
					if (cart.is(':visible')) {
						$('body,html').removeClass('no-scroll');
						$('body').removeClass('has-modal');
						cart.fadeOut(400, function() {
							if (c >= 0) {
								floatBtn.fadeIn(400);
							}
							if ($.isFunction(doneFn)) {
								doneFn.call(this);
							}
						});
					}
				};
			
		
		// Initializing
		
		init(loadData);
		
		// Handlers
		
			// Adding new items
		self.on('addItems', function(e) {
				var items = Array.prototype.slice.call(arguments, 1);
				addItems(items);
			})
			// Emptying cart
			.on('emptyCart', emptyCart)
			// Showing cart
			.on('showCart', showCart)
			// Hiding the cart
			.on('hideCart', hideCart);
		
		// Ready event
		
		if (settings.hasOwnProperty('onReady')) {
			settings.onReady(this);
		}

		return this;
		
	}
	
	$.fn.feedback.defaults = {
		expiry:					60 * 60 * 1000,		// 1 hour
		useFloatButton:			true,
		rootPath:				'',
		relPath:				'',
		formatting:				'human', 			// 'serialized' || 'human' || 'text'
		thumbsFolder:			'thumbs/',
		itemsStart:				'<!-- items:start -->',
		itemsStop:				'<!-- items:stop -->'
	};
		
	$.fn.feedback.text = getTranslations({
		edit:					'Edit',
		continueBrowsing:		'Continue browsing',
		feedback:				'Feedback',
		feedbackOnAlbum:		'Feedback on album "{0}"',
		closeWindow:			'Close window',
		emptyCart: 				'Empty cart',
		removeAllItems:			'Remove all items?',
		yes:					'Yes',
		no:						'No',
		overLimitWarning:		'This is too much to pass to the email application directly. Please copy / paste the text below to the email body!' 
	});
	
})(jQuery, $(window), $('body'));